/* eslint-disable @typescript-eslint/no-explicit-any */
import { containers, basicFields } from "@/utils/widgetsConfig";
import type { IWidget } from '@/types/widgetsConfigInterface'
/**
 * 初始化和重置文书结构
 * @param widgetJson
 */
export const initJson = (widgetJson: any) => {
  widgetJson['subDomList'] = []
  widgetJson['activeConfig']['activeId'] = ""
  widgetJson['activeConfig']['activeDisplayName'] = ""
}

/**
 * 根据表单元素 格式化对应的属性
 * @param addItemDataOrigin
 */
export const formatAddItem = (addItemTypeName: string) => {
  let addItemData = null;
  let activeId = ""
  let activeDisplayName = ""
  const allWidgetList = [...containers, ...basicFields];
  const targetItem = allWidgetList.filter(item => {
    return item['type'] == addItemTypeName
  })
  if (targetItem['length']) {
    addItemData = structuredClone(targetItem[0]);
    addItemData['key'] = `${Date.now().toString().slice(-6)}${Math.floor(Math.random() * 900) + 100}`;
    addItemData['id'] = `${addItemData['type']}-${addItemData['key']}`
    addItemData['subDomList'] = [];
    addItemData['options']['name'] = addItemData['id']
    activeId = addItemData['id']
    activeDisplayName = addItemData['displayName']
  }
  return {
    addItemData,
    activeId,
    activeDisplayName
  }
}

/**
 * 添加新的子元素
 * @param widgetJson 文书结构JSON
 * @param addItemTypeName 新元素类型
 * @param fatherId 父容器id
 */
export const addItem = (widgetJson: any, addItemTypeName: any) => {
  const { addItemData, activeId, activeDisplayName } = formatAddItem(addItemTypeName)
  if (widgetJson['activeConfig']['activeId']) { } else {
    widgetJson['subDomList'].push(addItemData)
  }
  widgetJson['activeConfig']['activeId'] = activeId
  widgetJson['activeDisplayName'] = activeDisplayName
}



/**
 * 根据id寻找结构
 * @param subDomList  widgetJson['subDomList']
 * @param returnParent 是否返回父容器节点
 */
export const handleFindDomById = (subDomList: any, activeKey: string, returnParent = false) => {
  let found: IWidget | null = null

  function dfs(nodes: IWidget[], parent: IWidget | null): boolean {
    for (const node of nodes) {
      if (node['id'] === activeKey) {
        found = returnParent ? parent?.category === 'container' ? parent : null : node
        return true
      }

      // 针对 subDomList 为二维数组或一维数组都做兼容处理
      const children = node.subDomList
      if (Array.isArray(children)) {
        if (Array.isArray(children[0])) {
          // 二维数组情况
          for (const row of children as IWidget[][]) {
            if (dfs(row, node)) return true
          }
        } else {
          // 一维数组情况
          if (dfs(children as IWidget[], node)) return true
        }
      }
    }
    return false
  }

  dfs(subDomList, null)
  return found
}